#ifndef ASTROMETRY_DEFS
#define ASTROMETRY_DEFS



#include "OmSourceParameters.h"


#include "Dal.h"






void computeSphericalCoordinates(double x, double y, double z, double &ra, double &dec);



/*****************************************************
*
* Computes the equatorial rectangular coordinates
*
*****************************************************/
class EquatorialCoordinates
{
public:
    double ra;
    double dec;
    double x;       
    double y;
    double z;
    EquatorialCoordinates() : ra(0.0), dec(0.0), x(0.0), y(0.0), z(0.0) {};

    EquatorialCoordinates(double raIn, double decIn);


    ~EquatorialCoordinates(){};

     
    void computeEquatorialCoordinates();
    
    /**************************************************************
    *
    **************************************************************/    
    void computeSphericalCoordinates();


    double arcLength( EquatorialCoordinates & a);


};    

class data : public EquatorialCoordinates {
public:

    data() : EquatorialCoordinates(0.0, 0.0), index(0), error(0.0), significance(0.0), extension(0) {};  
    data( int n, double a, double b, float e = 0.0, float s = 0.0, Integer1 k = 0) : 
      EquatorialCoordinates(a, b), index(n), error(e), significance(s), extension(k)  { };

    int index;    
    float error, significance;
    Integer1 extension;

};






class matchedData {
public:

    matchedData() : index(0), distance(0.0)  {};

    
    matchedData( int ind, double r) : 
      index(ind), distance(r)  {};
    int index;
    double distance;
 

};




/************************************************************************
*
*
*************************************************************************/
void averageCelestialCoordinates(vector<double> &ra, vector<double> &dec, 
				 double &raMean, double &decMean);




bool compare(const matchedData &a, const matchedData & b);
#endif



