module coords
use sla
use dal
use omconstants_mod
use ommagnitude_mod
use omfunctions_mod
interface convertToGalactic
    module procedure convertToGalactic1, convertToGalactic2

end interface convertToGalactic


contains


!*************************************************************
!
!*************************************************************


real(double) function chisqprobability(chisq, ndf)
    real(double), intent(in) :: chisq
    integer, intent(in) :: ndf


    ! Compute the probability
   
    chisqprobability = Gammaq(0.5_double * dble(ndf), 0.5_double * chisq)

   if(   chisqprobability > 100.0) then

    write(*, *) "chisq=", chisq, " ndf=", ndf, " prob=", chisqprobability

    call fatal("bug")
    end if

    

end function chisqprobability


!***********************************************************
! 
!**********************************************************/

subroutine converttoradec(x, y, ra0, dec0, ra, dec)
    real(double), intent(in) :: x, y, ra0, dec0
    real(double), intent(out) :: ra, dec


    call sla_dtp2s(x * degreesToRadians, y * degreesToRadians,  &
               ra0 * degreesToRadians, dec0 * degreesToRadians, &
               ra, dec)

!    ra = ra * radiansToDegrees
!    dec = dec * radiansToDegrees


end subroutine converttoradec








!********************************************************************
!
!********************************************************************
subroutine converttogalactic1(n, ra, dec, longitude, latitude)
    integer, intent(in) :: n
    real(double), intent(in), dimension(1 : n) :: ra, dec
    real(double), intent(out), dimension(1 : n) :: longitude, latitude
    integer :: i


    
    !*********************************************************************
    ! Compute galactic longitude and latitude of OM pointing direction
    !********************************************************************
    do i = 1, n
        call converttogalactic2(ra(i), dec(i), longitude(i), latitude(i))
    end do

end subroutine converttogalactic1
!**********************************************************************
!
!**********************************************************************

subroutine converttogalactic2(ra, dec, longitude, latitude)
    real(double), intent(in) :: ra, dec
    real(double), intent(out) :: longitude, latitude
    real(double) :: long, lat

    call sla_EQGAL( ra * degreesToRadians, dec * degreesToRadians, long, lat)
    longitude = long * radiansToDegrees
    latitude = lat   * radiansToDegrees
 

end subroutine converttogalactic2
!***********************************************************************
!
!***********************************************************************
subroutine degradation(date, factor)
    real(double), intent(in) :: date
    real(double), intent(out) :: factor

    factor = TDPCorrectionFactor( date, "V")

end subroutine degradation









!*******************************************************
! ModifiedJulianDate - Returns the modified Julian date
! from the string date representation
!*******************************************************
real(double) function modifiedjuliandate(year, month, day, hour, minute, second)
 

    integer, intent(in) :: year            ! Stores the year from the date string
    integer, intent(in) :: month           ! Stores the month from the date string
    integer, intent(in) :: day             ! Stores the day from the date string  
    integer, intent(in) :: hour            ! Stores the hour from the date string
    integer, intent(in) :: minute          ! Stores the minute from the date string  
    integer, intent(in) :: second          ! Stores the second from the date string
    
    ! from string representation
    real(double) JulianDay    
    integer :: flag



    ! Get the modified Julian date for this date


    call sla_CLDJ(year, month, day, JulianDay, flag)


!    write(*, *) "modifiedjuliandate- julinaday=", julianDay
!    write(*, *) "hour=", hour, " minute=",  minute, " second=", second
    ! Need to add on the fraction of a day corresponding to the time
    ModifiedJulianDate = JulianDay + ( dble(hour) + &
         ( dble(minute) + dble(second) / 60.0d0 ) / 60.0d0 ) / 24.0d0



end function modifiedjuliandate










end module coords
