#include "Dal.h"
#include <iostream>
#include "SasDalUtils.h"
#include "dal_utilities.h"
#include "DataSubspace.h"
#include<cstdlib>
#include<vector>
#include <map>
#include<cmath>
#include<cstring>
#include "f90cxx.h"
#include<numeric>
#include<algorithm>

F90MODPROCPROTO( double, coords, chisqprobability)(double *, int *);



/******************************************************
*
*******************************************************/
bool LoadVectorFloatColumnData(const Table *table, 
                      const string& columnName, 
		      int &size, vector<float> &data);

bool LoadVectorDoubleColumnData(const Table *table, 
                      const string& columnName, 
		      int &size, vector<double> &data);
void loadData(string file, vector<double> &ra, vector<double> &dec);


void setColumnData( const int size, const ColumnData *columnData, const vector<real32>& data );
void setColumnData( const int size, const ColumnData *columnData, const vector<real64>& data );
void setColumnData( const int size, const ColumnData *columnData, const vector<int8>& data );

void setColumnData( const int size, const ColumnData *columnData, const vector<int16>& data );
void setColumnData( const int size, const ColumnData *columnData, const vector<int32>& data );



/*****************************************************************************
*
*****************************************************************************/ 
void createTimeString(int hour, string &dateString);
   




bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<real32> &data, int position = 0);

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<real64> &data, int position = 0);

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<int8> &data, int position = 0);


bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<int16> &data, int position = 0);


bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	             vector<int32> &data, int position = 0);




void sortTable( string file );
void sortTable( Table **tableH );







/*******************************************************
*
*
*
********************************************************/
class OMData
{
public:
    OMData() {};
    OMData(const OMData &c)
    {
      _number = c._number;
        _ra = c._ra;
        _dec = c._dec;
        _significance = c._significance;
        _rawRate = c._rawRate;
        _rawRateError = c._rawRateError;

        _corrRate = c._corrRate;
        _corrRateError = c._corrRateError;

        _mag = c._mag;
        _magError = c._magError;
        _eFlag = c._eFlag;
        _filter = c._filter;
        _number = c._number;
    }
    vector<int> _number;
    vector<double> _ra, _dec;
    vector<float> _significance;
    vector<float> _rawRate, _corrRate;
    vector<float> _rawRateError, _corrRateError;

    vector<float> _mag, _magError;
    vector<int8> _eFlag;
    string _filter;
   

};
/******************************************************************
*
*
*******************************************************************/


class VariabilityData
{
public:
  VariabilityData() : _chiSquared(0), _nstd(0) {};
  VariabilityData(const VariabilityData &c)
  {
    _chiSquared = c._chiSquared;
    _nstd = c._nstd;
  }

  float _chiSquared;
  float _nstd;       

};



void addVariabilityColumns( string outputFile, 
                           string filter,                        
                           map<int, VariabilityData> &varData);  



void createOutputFile(string file, map<string, OMData>  &data);

void createMasterFile(string masterFile, int &numFiles, vector<OMData> &data );

void createSWSFile(string file, OMData &data, int hour);


void computeVariabilityStatistics( map<int, OMData> &dataMap, map<int, VariabilityData> & );

float computeChiSquared(vector<float> &x, vector<float> &dx, float &theMean, float &prob);

class OMData1
{
public:
  OMData1() {};
  OMData1(int n, double ra): _number(n), _ra(ra) {};


  OMData1(const OMData1 &c)
  {
    _number = c._number;
    _ra = c._ra;

  }
  int _number;
  double _ra;
   
};
/************************************************************
*
************************************************************/
bool compare(const OMData1 &a, const OMData1 & b)
{
    return a._ra < b._ra;

};




 
/**********************************************************************
*
*
*
*
*
*
**********************************************************************/ 

int main()
{

    srand ( time(NULL) );


    /*******************************************************
    *
    * Create 4 OWS files
    *
    ********************************************************/

    int numFiles = 3 + rand() % 7;
 
    numFiles = 3;


    vector<OMData> data(numFiles * 3);
 

    cout << "createfiles line 236\n";
    createMasterFile("masterfile.fits", numFiles, data);

    cout << "After creating the master file the number of SWS files is " << numFiles << endl;
  
    
    for(int i = 0; i < numFiles; i++)
    {


        char file[100];
        sprintf(file, "test%0d.fits", i);

        int hour = i;   

        cout << i << ") Creating SWS file- filter=" << data[i]._filter << " time=" << hour << endl;  
        createSWSFile( file, data[i], hour );
      
        sortTable( file );


    }


    cout << "Finished\n";
    return 0;

}

/***********************************************************
*
*
*
*
**********************************************************/
void loadData(string file, vector<double> &ra, vector<double> &dec)
{

    DataSet *dataSet = dataSetServer->open( file, Dal::Read);

    if (!dataSet)
    {


    }    
 
    const Table *table = dataSet->table( "SRCLIST", Dal::Read);
    if( !table) {};

    /**********************************************************************
    *
    *
    **********************************************************************/
  
    int size = 0;

 
    if ( !LoadVectorDoubleColumnData( table, "RA", size, ra))
    {

    

    }

 
    if ( !LoadVectorDoubleColumnData( table, "DEC", size, dec))
    {

    

    }

}
/********************************************************
*
*
******************************************************/
bool LoadVectorFloatColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<float> &data)
{
     
    size = table->rows();
   
    if (table->hasColumn(columnName))
    { 

        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
   
    return true;

};

/********************************************************
*
*
******************************************************/
bool LoadVectorDoubleColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<double> &data)
{
       
    size = table->rows();
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
   
    return true;
};



/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real32> &data, int position)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real32, comment, units, size, position);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};



/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real64> &data, int position)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real64, comment, units, size, position);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};

/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int8> &data, int position)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int8, comment, units , size, position);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};



/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int16> &data, int position)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int16, comment, units , size, position);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};



/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int32> &data, int position)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Int32, comment, units , size, position);
 
    ColumnData *columnData = column->data();

    setColumnData( ( int)size[1], columnData, data);


    delete columnData;


          
    return true;
};




/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int8>& data)
{
    int8 * r64 = columnData->int8Data();
    for(int i = 0; i < size; i++)
        r64[ i] = data[i];


}
/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int16>& data)
{
    int16 * r64 = columnData->int16Data();
    for(int i = 0; i < size; i++)
        r64[ i] = data[i];


}
/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<int32>& data)
{
    int32 * r64 = columnData->int32Data();
    for(int i = 0; i < size; i++)
        r64[ i] = data[i];


}

/******************************************************
*
******************************************************/



void setColumnData( const int size, const ColumnData *columnData, const vector<real32> &data)
{
  

    real32 * r64 = columnData->real32Data();
    for(int i = 0; i < size; i++)
    {
     
        if( abs( data[i] + 9999.9) < 1e-3)
            columnData -> setToNull( i );
        else 

            r64[i] = data[i];
    }

}



/******************************************************
*
******************************************************/
void setColumnData( const int size, const ColumnData *columnData, const vector<real64> &data)
{
  

    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
    {
     
        if( abs( data[i] + 9999.9) < 1e-3)
            columnData -> setToNull( i );
        else 

            r64[i] = data[i];
    }

}


/*****************************************************************************
*
*
*
******************************************************************************/
void createOutputFile(string file, map<string, OMData> &data)
{


    DataSet *dataSet = dataSetServer->open(file, Dal::Create);

    *dataSet->addAttribute( "FILTER", "" ) = "n/a";

    *dataSet->addAttribute( "OBS_ID", "" ) = "666";

    if (!dataSet) return ;    


 
    


    string filter = "U";

    Table *table = dataSet->addTable( "SRCLIST", data[filter]._ra.size() );

    if (!addColumn( table, "RA", "degrees", "", 
		     (int)data[filter]._ra.size(), data[filter]._ra) )
    {
     
    }

 


    if (!addColumn( table, "DEC", "degrees", "",
		     (int)data[filter]._dec.size(), data[filter]._dec) )
    {
     
    }


    if (!addColumn( table, "SRCID", "", "",
		     (int)data[filter]._ra.size(), data[filter]._number) )
    {
     
    }


    /***********************************************************************
    *
    * Luoop through all th filters
    *
    ************************************************************************/


    map<string, OMData>::iterator iter;

    for(iter = data.begin(); iter != data.end(); ++iter)
    {
       
        string filter = iter->first;;
        OMData data = iter->second;

        string name = filter + "_SIGNIFICANCE";
        if (!addColumn( table, (const char *)name.c_str(), "", "",
    	     (int)data._ra.size(), data._significance) )
        {
     
        }



        name = filter + "_CORR_RATE";
        if (!addColumn( table, (const char*)name.c_str(), "", "",
		     (int)data._ra.size(), data._corrRate) )
        {
     
        }


        name = filter + "_CORR_RATE_ERR";
        if (!addColumn( table, (const char*)name.c_str(), "", "",
		     (int)data._ra.size(), data._corrRateError) )
        {
     
        }



        name = filter + "_EXTENDED_FLAG";
 

        if (!addColumn( table, (const char*)name.c_str(), "degrees", "",
		     (int)data._ra.size(), data._eFlag) )
        {
     
        }


    } 



    dataSetServer->close( dataSet );

 
 
}







/*****************************************************************************
*
*
*
******************************************************************************/
void createSWSFile(string file, OMData &data, int hour)
{

 
 
    DataSet *dataSet = dataSetServer->open(file, Dal::Create);

   *dataSet->addAttribute( "FILTER", "" ) = data._filter;

   *dataSet->addAttribute( "OBS_ID", "" ) = "666";

    /*****************************************************************************
    *
    *****************************************************************************/ 
    string dateString;
    createTimeString(hour, dateString);

   



    *dataSet->addAttribute( "DATE-OBS", "" ) = dateString;
   createTimeString(hour+1, dateString);

   *dataSet->addAttribute( "DATE-END", "" ) = dateString;
   *dataSet->addAttribute( "OBJECT", "" )   = "NGC 5128";




    if (!dataSet) return ;    


   




    Table *table = dataSet->addTable( "SRCLIST", data._ra.size() );

    if (!addColumn( table, "RA", "degrees", "", 
		     (int)data._ra.size(), data._ra) )
    {
     
    }

  

 



    if (!addColumn( table, "DEC", "degrees", "",
		     (int)data._dec.size(), data._dec) )
    {
     
    }

  





    string name = "SIGNIFICANCE";
    if (!addColumn( table, (const char *)name.c_str(), "", "",
    	     (int)data._ra.size(), data._significance) )
    {
     
    }
  


 



    name = "CORR_RATE";
    if (!addColumn( table, (const char*)name.c_str(), "", "",
		     (int)data._ra.size(), data._corrRate) )
    {
     
    }



 



    name = "CORR_RATE_ERR";
    if (!addColumn( table, (const char*)name.c_str(), "", "",
		     (int)data._ra.size(), data._corrRateError) )
    {
     
    }



 



    name = "MAG";
    if (!addColumn( table, (const char*)name.c_str(), "", "",
		     (int)data._ra.size(), data._mag) )
    {
     
    }





    name = "MAGERR";
    if (!addColumn( table, (const char*)name.c_str(), "", "",
		     (int)data._ra.size(), data._magError) )
    {
     
    }



 



 
    name = "EFLAG";
    if (!addColumn( table, (const char*)name.c_str(), "", "",
		     (int)data._ra.size(), data._eFlag) )
    {
     
    }






    name = "SRC_ID";
    if (!addColumn( table, (const char*)name.c_str(), "", "",
		     (int)data._ra.size(), data._number) )
    {
     
    }


 




    if(table){};
    dataSetServer->close( dataSet );

 
}

/*****************************************************************
*
*
*
*****************************************************************/
void createMasterFile(string masterFile, int &numFiles, vector<OMData> &data)
{


    /*****************************************************
    *
    * Create the observation source-list file
    *
    ******************************************************/
    map<string, OMData> masterData;

    masterData["U"]._filter = "U";
    masterData["B"]._filter = "B";
    masterData["V"]._filter = "V";

    
    cout << "Createmasterfile line 944\n";


    /*************************************************
    *
    *
    * The total number of sources in the master file
    *
    *
    *************************************************/
    int size = max(20, rand() % 100);

    double ra0 = rand() % 360;
    double dec0 = -20.0 + rand() % 140;
    double da = 10.0 / 3600.0;
    double ddec = 10.0 / 3600.0;

  
    
    for(int i = 0; i < size; i++)
    {

        double ra = ra0 + (double)i * da;
        double dec = dec0 + (double)i * ddec;
        float sig = 2.0 + rand() % 1000;

        /*****************************************************
	*
        * Loop through all the filters
        *
        ******************************************************/

        map<string, OMData>::iterator iter;
        for (iter = masterData.begin(); iter != masterData.end(); ++iter)
	{

	    string filter = iter->first;

            masterData[filter]._number.push_back( i + 1);
            masterData[filter]._ra.push_back( ra );
            masterData[filter]._dec.push_back( dec );
    
 
            masterData[filter]._significance.push_back( sig );
       
            float rate = sig;
	    float rateError = 0.1 * sig;
            masterData[filter]._rawRate.push_back( rate );
       
            masterData[filter]._corrRate.push_back( rate );
        
    
            masterData[filter]._corrRateError.push_back( rateError );
	    float mag = -2.5 * log10( rate ) + 19.0;
            masterData[filter]._mag.push_back( mag );

	    float magError = rateError / rate;
            masterData[filter]._magError.push_back( magError );
            masterData[filter]._eFlag.push_back( (int8)1 );

        }
    }

 cout << "Createmasterfile line 1007\n";

 
    /****************************************************
    *
    * Create the master observation source-list file
    *
    *****************************************************/
    createOutputFile(masterFile, masterData);
    createOutputFile("checkfile.fits", masterData);



 cout << "Createmasterfile line 1020\n";


    /*************************************************
    *
    * The data for the other files
    *
    *************************************************/


    map<string, OMData>::iterator iter;
    int fileNum = 0;
    for (iter = masterData.begin(); iter != masterData.end(); ++iter) // Looping through the filters
    {
        string filter = iter->first;
        map<int, OMData> dataMap;

	cout << "Createmasterfile line 1037 filter=" << filter << endl;


        for(int i = 0; i < numFiles; i++)  // Looping through the source-list files for this filter 
        {



            data[fileNum]._filter = filter;
	    cout << "Createmasterfile line 1046\n";

            for(int j = 0; j < size; j++)        // Loop through all the master sources
	    {

	        int k = rand() % 100;
                if(k > 70) continue;  

                /*****************************************************************
		* NB data is for a SWS file
                *
                ******************************************************************/
      
         
                data[fileNum]._number.push_back( masterData[filter]._number[j] );
       
                data[fileNum]._ra.push_back( masterData[filter]._ra[j] );
                data[fileNum]._dec.push_back( masterData[filter]._dec[j] );
                data[fileNum]._significance.push_back( masterData[filter]._significance[j] );
    
                data[fileNum]._rawRate.push_back( masterData[filter]._rawRate[j] );
                data[fileNum]._corrRate.push_back( masterData[filter]._corrRate[j] );
                data[fileNum]._corrRateError.push_back( masterData[filter]._corrRateError[j] );

                data[fileNum]._mag.push_back( masterData[filter]._mag[j] );
                data[fileNum]._magError.push_back( masterData[filter]._magError[j] );
 
                data[fileNum]._eFlag.push_back( masterData[filter]._eFlag[j] );

       
                /*****************************************************************
	        *
                * For computing variability statistics
                *
                *******************************************************************/
     
                dataMap[j]._ra.push_back( masterData[filter]._ra[j] );
                dataMap[j]._dec.push_back( masterData[filter]._dec[j] );
                dataMap[j]._significance.push_back( masterData[filter]._significance[j] );
    
                dataMap[j]._rawRate.push_back( masterData[filter]._rawRate[j] );
                dataMap[j]._corrRate.push_back( masterData[filter]._corrRate[j] );
                dataMap[j]._corrRateError.push_back( masterData[filter]._corrRateError[j] );

                dataMap[j]._mag.push_back( masterData[filter]._mag[j] );
                dataMap[j]._magError.push_back( masterData[filter]._magError[j] );
 
                dataMap[j]._eFlag.push_back( masterData[filter]._eFlag[j] );
                dataMap[j]._number.push_back( masterData[filter]._number[j] );


            }
    
            ++fileNum; // Move on to the next source-list file

	    cout << "Line 1098 filenum=" << fileNum << endl;
     

	} // End of looping through the number of files for this filter
 
        map<int, VariabilityData> variabilityMap;
        computeVariabilityStatistics( dataMap, variabilityMap );


        /***************************************************************
        *
        *
        ****************************************************************/

        addVariabilityColumns( "checkfile.fits",                 
                           masterData[filter]._filter,                         // The filter
                           variabilityMap);                             // The list of variability parameters for each SRCID  
                           




    
    } // End of looping through the filters

    /*********************************************************************************
    *
    * checkfile.fits is the same as masterfile with the variability parameters added
    *
    **********************************************************************************/


    cout << "Line 1122 numFiles=" << numFiles << " fileNum=" << fileNum << endl;    

    numFiles = fileNum;


}

/*************************************************************************
*
*
**************************************************************************/
void sortTable( string file )
{


    DataSet *dataSet = dataSetServer->open(file, Dal::Modify);
    if (!dataSet) return ;    


    

    Table *tableH = dataSet->table("SRCLIST");
  
    sortTable( &tableH );


    dataSetServer->close(dataSet); // Finished with the file





}
/*******************************************************************
*
*
*
********************************************************************/
void sortTable( Table **tableH )
{
 
    int nRows = (*tableH)->rows();


    if(nRows < 2)
    {
        return;
    }


    /**********************************************************
    * 
    *
    *********************************************************/
    vector<double> ra;

    if( (*tableH)->hasColumn("RA_CORR") ) 
    {
        int size = 0; 
        if ( !LoadVectorDoubleColumnData( *tableH, "RA_CORR", size, ra))
	{
        }
    }
    else
    {
        int size = 0;
        if ( !LoadVectorDoubleColumnData( *tableH, "RA", size, ra) )
	{
        }


    }

    vector<OMData1> data;

    for(int i = 0; i < nRows; i++)
    {

        
        data.push_back( OMData1(i, ra[i]) );
       
  

    }

    vector<int> raSortIndex(nRows);   

 
    /*****************************************************
    * Find the order of increasing  RA- store in RASortIndex
    * for adding data to source list table in  
    * increasing RA order (ra is not sorted)
    *****************************************************/
    sort( data.begin(), data.end(), compare);

    for(int i = 0; i < nRows; i++)
    {
 
        raSortIndex[i] = data[i]._number; 
    }

    //    cout << "Sorttable line the sorted data 74\n";

    //  for(int i = 0; i < nRows; i++)
    //   cout << i << ") ra=" << sourceList[i].rightascension << " index=" << raSortIndex[i] << endl;



    (*tableH)->insertRows( nRows,  nRows );
    (*tableH)->copyRows( 0, nRows, nRows );

    /*************************************************************
    * Reorder the table into increasing RA order
    **************************************************************/
  

    for(int i = 0; i <  nRows; i++)
    {
        int k = raSortIndex[i] + nRows;       
	//	cout << i << ") k=" << k << " rasortindex=" << raSortIndex[i] << endl;
        (*tableH)->copyRows(k, i, 1);
    }


    /********************************************************
    *
    * Delete the appended temporary rows
    *
    *********************************************************/


    (*tableH)->deleteRows( nRows, nRows);
 
    



}
/******************************************************************
*
* Compute the variability statistics
*
*****************************************************************/

void computeVariabilityStatistics( map<int, OMData> &dataMap, map<int, VariabilityData> &chiSquaredMap )
{

    map<int, OMData>::iterator iter;

    for(iter = dataMap.begin(); iter != dataMap.end(); ++iter)
    {

        int srcNum = iter->first;
        OMData data = iter->second;

        if( data._ra.size() < 2) continue;
      
        float mean = 0.0, prob = 0.0;
        float chiSquared = computeChiSquared( data._corrRate, data._corrRateError, mean, prob);

	chiSquaredMap[srcNum]._chiSquared = chiSquared;

    }



}
/*******************************************************************************
*
*
* Compute the chi-squared
*
*
*********************************************************************************/
float computeChiSquared(vector<float> &x, vector<float> &dx, float &theMean, float &prob)
{

    /**********************************************************
    * Compute the weighted mean
    ***********************************************************/

    double mean = (double)0.0;
    vector<double> weight(x.size() );
    double sumWeight = (double)0.0;

    for(unsigned i = 0; i < x.size(); i++)
    {
        double w = (double)1.0 / ( double(dx[i]) * double(dx[i]));
        weight[i] = w;
        mean += double(x[i]) * w;     
        sumWeight += w;
      
    }

    mean /= sumWeight;
    
    double chiSquared = (double)0.0;

    for(unsigned i = 0; i < x.size(); i++)
    {
        double diff = x[i] - mean;
        chiSquared += diff * diff * weight[i];

    }
    theMean = (float)mean;

 

    int ndf = x.size() - 1;
    prob = F90MODPROC( coords, chisqprobability)( &chiSquared, &ndf);


         
    return float( chiSquared / double(x.size() - 1) );
 

}

/******************************************************************
*
*
* Adds the variability columns for a given filter
* size is the number of rows in the mastercatalogue
*
*
*******************************************************************/
void addVariabilityColumns( string outputFile, 
                            string filter,                         // The filter
                            map<int, VariabilityData> &varData)    // The list of variability parameters for each SRCID  
                           
{

 
    cout << "Adding variability columns for filter " << filter << endl;

    /**************************************************************
    *
    * Add the columns to the output catalogue
    *
    **************************************************************/


    /******************************************************************
    *
    * 
    *
    *******************************************************************/
    DataSet * set = dataSetServer->open( outputFile, DataSet::Modify);
    Table *table = set->table("SRCLIST");

    int size = table->rows();



    /********************************************************************
    *
    * Need to load the SRCID column from the OBS file
    *
    ********************************************************************/
    //  vector<int> srcId;
    //    LoadColumnData( table,  "SRCID", srcId);
    //    dataSetServer->close( set); // Finished with the file



   




    vector<float> chi(size, -9999.9);     // For chi-squared in output catalogue file
    vector<float> maxDev(size, -9999.9);  // for MAXDEV




    map<int, VariabilityData>::iterator iter;
    for(iter = varData.begin(); iter != varData.end(); ++iter)
    {
        int i = iter->first; // NB 0-size - 1
        
        VariabilityData data = iter->second;
        chi[i] = data._chiSquared;
        maxDev[i] = data._nstd;       

    }


    string columnName = filter + "_CORR_RATE_ERR"; 
  
    int position = table->columnNumber( columnName ) + 1; 
 

    columnName = filter + "_CHI2"; 


    string units = "";
    string comment = "Variability chi**2 statistic (" + filter + " filter)";


    /********************************************************
    *
    *
    ******************************************************/


    if (!addColumn( table,   
                    (char *)columnName.c_str(), 
                    (char *)units.c_str(),
                    (char *)comment.c_str(),   
	            size,
	            chi,
                    position) )
    {

    }
    ++position;
    columnName = filter + "_MAXDEV"; 
    cout << "Adding column " << columnName << endl;


    comment = "Max. dev. from median count-rate (" + filter + " filter)";

    units = "sigma";


    /********************************************************
    *
    *
    ******************************************************/
    if (!addColumn( table, 
                    (char *)columnName.c_str(), 
                    (char *)units.c_str(),
                    (char *)comment.c_str(),   
	            size,
	            maxDev,
                    position + 1))
    {

    }


    cout << "Added column " << columnName << endl;    


   dataSetServer->close( set); // Finished with the file


}

/*****************************************************************************
*
*****************************************************************************/ 
void createTimeString(int hour, string &dateString)
{
   
   
    int year = 2000, month = 1, day = 1, min = 0;
    double sec = 0.0;
    char text[100];
 

    sprintf( text, "%4d%c%02d%c%02d%c%02d%c%02d%c%05.2f", 
                      year, '-', month, '-', day, 'T', hour, ':', min, ':', sec); 

    cout << "datestring=" << text << endl;

    dateString = text;    
}




